<?php
declare(strict_types = 1);
namespace Drupal\set_author\Plugin\EntityShareClient\Processor;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\entity_share\EntityShareUtility;
use Drupal\entity_share_client\ImportProcessor\ImportProcessorPluginBase;
use Drupal\entity_share_client\RuntimeImportContext;
use Drupal\user\Entity\User;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Set the author for a shared node.
 *
 * @ImportProcessor(
 *   id = "set_node_author",
 *   label = @Translation("Set node author"),
 *   description = @Translation("Set the author of shared nodes to a pre-configured author."),
 *   stages = {
 *     "process_entity" = 110,
 *   },
 *   locked = false,
 * )
 */
class SetAuthor extends ImportProcessorPluginBase implements PluginFormInterface {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The remote manager.
   *
   * @var \Drupal\entity_share_client\Service\RemoteManagerInterface
   */
  protected $remoteManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $entity_type_manager = $container->get('entity_type.manager');
    $instance->entityTypeManager = $entity_type_manager;
    $instance->logger = $container->get('logger.channel.entity_share_client');
    $instance->remoteManager = $container->get('entity_share_client.remote_manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $default_user = User::getAnonymousUser();
    return [
        'shared_node_author' => $default_user->id(),
      ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $shared_node_author = User::load($this->configuration['shared_node_author']);
    $form['shared_node_author'] = [
      '#title' => $this->t('Shared node author'),
      '#description' => $this->t(
        'This user will be used as the author of shared nodes by default if source author not exist in this site.'
      ),
      '#type' => 'entity_autocomplete',
      '#required' => TRUE,
      '#target_type' => 'user',
      '#default_value' => $shared_node_author,
      '#selection_settings' => [
        'include_anonymous' => TRUE,
      ],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function processEntity(RuntimeImportContext $runtime_import_context,
    ContentEntityInterface $processed_entity,
    array $entity_json_data) {
    $entity_type_id = $processed_entity->getEntityTypeId();
    if ($entity_type_id == 'node') {
      $field_data = $entity_json_data['relationships']['uid'];
      // Check that the field has data.
      if ($field_data['data'] != NULL) {
        $prepared_field_data = EntityShareUtility::prepareData($field_data['data']);

        // Get author id if exist.
        if ($this->getExistingEntities($prepared_field_data)) {
          // Get author id by uuid.
          $author_id = $this->getExistingEntities($prepared_field_data);
        }
        elseif (isset($field_data['links']['related']['href'])) {
          // Get author id by mail.
          $user_attributes = $this->importUrl($runtime_import_context, $field_data['links']['related']['href']);
          $user = user_load_by_mail($user_attributes['mail']);
          if ($user) {
            $author_id = $user->id();
          }
        }
        else {
          // Set default author id.
          $author_id = $this->configuration['shared_node_author'];
        }

      }
      if ($processed_entity->uid !== $author_id) {
        $processed_entity->uid = $author_id;
      }
    }
  }

  /**
   * Helper function to get existing uid.
   *
   * @param array $data
   *   The JSON:API data for user.
   *
   * @return string|bool
   *   uid of existing user or False.
   */
  protected function getExistingEntities(array $data) {

    if ($data['id'] !== 'missing') {
      try {
        $entity_storage = $this->entityTypeManager->getStorage('user');
        $existing_entity_id = $entity_storage->getQuery()
          ->condition('uuid', $data['id'], '=')
          ->execute();

        if (!isset($existing_entity_id)) {
          return reset($existing_entity_id);
        }
      }
      catch (\Exception $e) {
        $log_variables = [];
        $log_variables['@msg'] = $e->getMessage();
        $this->logger->error('Caught exception trying to load existing entities. Error message was @msg', $log_variables);
      }
    }
    return FALSE;
  }

  /**
   * Helper function.
   *
   * @param \Drupal\entity_share_client\RuntimeImportContext $runtime_import_context
   *   The runtime import context.
   * @param string $url
   *   The URL to import.
   *
   * @return array
   *   The user attributes.
   */
  protected function importUrl(RuntimeImportContext $runtime_import_context, $url) {
    $json = $this->remoteManager->jsonApiRequest($runtime_import_context->getRemote(), 'GET', $url);
    $body = Json::decode((string) $json->getBody());

    // $body['data'] can be null in the case of
    // missing/deleted referenced entities.
    if (!isset($body['errors']) && !is_null($body['data'] && !isset($body['data']['attributes']))) {
      return $body['data']['attributes'];
    }

    return FALSE;
  }

}
